#import "DrumsViewController.h"

void* GetOpenALAudioData(
        CFURLRef fileURL, ALsizei* dataSize, ALenum* dataFormat, ALsizei *sampleRate)
{
    OSStatus    err;
    UInt32      size;
    
    // オーディオファイルを開く
    ExtAudioFileRef audioFile;
    err = ExtAudioFileOpenURL(fileURL, &audioFile);
    if (err) {
        goto Exit;
    }
    
    // オーディオデータフォーマットを取得する
    AudioStreamBasicDescription fileFormat;
    size = sizeof(fileFormat);
    err = ExtAudioFileGetProperty(
            audioFile, kExtAudioFileProperty_FileDataFormat, &size, &fileFormat);
    if (err) {
        goto Exit;
    }
    
    // アウトプットフォーマットを設定する
    AudioStreamBasicDescription outputFormat;
    outputFormat.mSampleRate = fileFormat.mSampleRate;
    outputFormat.mChannelsPerFrame = fileFormat.mChannelsPerFrame;
    outputFormat.mFormatID = kAudioFormatLinearPCM;
    outputFormat.mBytesPerPacket = 2 * outputFormat.mChannelsPerFrame;
    outputFormat.mFramesPerPacket = 1;
    outputFormat.mBytesPerFrame = 2 * outputFormat.mChannelsPerFrame;
    outputFormat.mBitsPerChannel = 16;
    outputFormat.mFormatFlags = kAudioFormatFlagsNativeEndian | kAudioFormatFlagIsPacked | kAudioFormatFlagIsSignedInteger;
    err = ExtAudioFileSetProperty(
            audioFile, kExtAudioFileProperty_ClientDataFormat, sizeof(outputFormat), &outputFormat);
    if (err) {
        goto Exit;
    }
    
    // フレーム数を取得する
    SInt64  fileLengthFrames = 0;
    size = sizeof(fileLengthFrames);
    err = ExtAudioFileGetProperty(
            audioFile, kExtAudioFileProperty_FileLengthFrames, &size, &fileLengthFrames);
    if (err) {
        goto Exit;
    }
    
    // バッファを用意する
    UInt32          bufferSize;
    void*           data;
    AudioBufferList dataBuffer;
    bufferSize = fileLengthFrames * outputFormat.mBytesPerFrame;;
    data = malloc(bufferSize);
    dataBuffer.mNumberBuffers = 1;
    dataBuffer.mBuffers[0].mDataByteSize = bufferSize;
    dataBuffer.mBuffers[0].mNumberChannels = outputFormat.mChannelsPerFrame;
    dataBuffer.mBuffers[0].mData = data;
    
    // バッファにデータを読み込む
    err = ExtAudioFileRead(audioFile, (UInt32*)&fileLengthFrames, &dataBuffer);
    if (err) {
        free(data);
        goto Exit;
    }
    
    // 出力値を設定する
    *dataSize = (ALsizei)bufferSize;
    *dataFormat = (outputFormat.mChannelsPerFrame > 1) ? AL_FORMAT_STEREO16 : AL_FORMAT_MONO16;
    *sampleRate = (ALsizei)outputFormat.mSampleRate;
    
Exit:
    // オーディオファイルを破棄する
    if (audioFile) {
        ExtAudioFileDispose(audioFile);
    }
    
    return data;
}

@implementation DrumsViewController

- (void)viewDidLoad
{
    // OpneALデバイスを開く
    ALCdevice*  device;
    device = alcOpenDevice(NULL);
    
    // OpenALコンテキスを作成して、カレントにする
    ALCcontext* alContext;
    alContext = alcCreateContext(device, NULL);
    alcMakeContextCurrent(alContext);
    
    // バッファとソースを作成する
    alGenBuffers(8, _buffers);
    alGenSources(8, _sources);
    
    int i;
    for (i = 0; i < 8; i++) {
        // サウンドファイルパスを取得する
        NSString*   fileName = nil;
        NSString*   path;
        switch (i) {
        case 0: fileName = @"BD"; break;
        case 1: fileName = @"SD"; break;
        case 2: fileName = @"FTom"; break;
        case 3: fileName = @"MTom"; break;
        case 4: fileName = @"HTom"; break;
        case 5: fileName = @"HH"; break;
        case 6: fileName = @"CCym"; break;
        case 7: fileName = @"Ride"; break;
        }
        path = [[NSBundle mainBundle] pathForResource:fileName ofType:@"m4a"];
        
        // オーディオデータを取得する
        void*   audioData;
        ALsizei dataSize;
        ALenum  dataFormat;
        ALsizei sampleRate;
        audioData = GetOpenALAudioData(
                (CFURLRef)[NSURL fileURLWithPath:path], &dataSize, &dataFormat, &sampleRate);
        
        // データをバッファに設定する
        alBufferData(_buffers[i], dataFormat, audioData, dataSize, sampleRate);
        
        // バッファをソースに設定する
        alSourcei(_sources[i], AL_BUFFER, _buffers[i]);
    }
}

- (IBAction)playSound:(id)sender
{
    // ドラムインデックスを取得する
    int index;
    index = [sender tag] - 100;
    
    // オーディオを再生する
    alSourcePlay(_sources[index]);
}

@end
